/*
 * requestUtils.java
 *
 * Created on 03 March 2004, 15:11
 */

package jim.util;

import java.security.Principal;
import java.util.Enumeration;
import javax.servlet.http.HttpServletRequest;
import org.w3c.tools.codec.Base64Decoder;
import org.w3c.tools.codec.Base64FormatException;

/**
 *
 * @author  briggsj
 */
public class RequestUtils {
    
    /** Creates a new instance of requestUtils */
    public RequestUtils() {
    }
    
    public static String getParameter(HttpServletRequest request, String name, String dflt) {
        if (request == null) {
            return dflt;
        } else {
            String s = request.getParameter(name);
            if (s == null) {
                return dflt;
            } else {
                return s.trim();
            }
        }
    }
    
    public static int getParameter(HttpServletRequest request, String name, int dflt) {
        if (request == null) {
            return dflt;
        } else {
            String s = request.getParameter(name);
            if (s == null) {
                return dflt;
            }
            if (s.length() == 0) {
                return dflt;
            }
            try {
                int n = Integer.parseInt(s);
                return n;
            } catch (NumberFormatException e) {
                return dflt;
            }
        }
    }
    
    public static String getParameter(HttpServletRequest request, String name) {
        return getParameter(request, name, "");
    }
    
    public static String getDebugString(HttpServletRequest request) {
        StringBuffer buf = new StringBuffer();
        
        for (java.util.Enumeration e = request.getAttributeNames(); e.hasMoreElements() ;) {
            buf.append("<BR>"+e.nextElement());
        }
        return buf.toString();
    }
    
    public static String displayAsHtml(HttpServletRequest request) {
        StringBuffer sb = new StringBuffer();
        String sep = "<BR>\n";
        
        //Request
        sb.append(request.getRequestURL());
        sb.append("?" + request.getQueryString());
        sb.append(sep);
        //Headers
        sb.append("Headers:" + sep);
        Enumeration e = request.getHeaderNames();
        if (e == null) {
            sb.append("No access to headers" + sep);
        } else {
            while (e.hasMoreElements()) {
                String header = (String)e.nextElement();
                Enumeration hdrs = request.getHeaders(header);
                while (hdrs.hasMoreElements()) {
                    sb.append(header + ": " + hdrs.nextElement());
                    sb.append(sep);
                }
            }
        }
        //Params
        sb.append("Parameters:" + sep);
        Enumeration paramNames = request.getParameterNames();
        while (paramNames.hasMoreElements()) {
            String param = (String)paramNames.nextElement();
            String[] values = request.getParameterValues(param);
            if (values == null || values.length < 1) {
                sb.append(param + "=");
                sb.append(sep);
            } else {
                for (int n = 0; n<values.length; n++) {
                    sb.append(param + "=" + values[n]);
                    sb.append(sep);
                }
            }
        }
        //Details
        sb.append("Details:" + sep);
        sb.append("Method: " + request.getMethod() + sep);
        sb.append("Authtype: " + request.getAuthType() + sep);
        Principal p = request.getUserPrincipal();
        if (p == null) {
            sb.append("User Principal: null" + sep);
        } else {
            sb.append("User Principal: " + p.getName() + sep);
        }
        sb.append("Authorized user: " + getAuthName(request) + sep);
        sb.append("Remote host: " + request.getRemoteHost() + sep);
        sb.append("Remote addr: " + request.getRemoteAddr() + sep);
        return sb.toString();
    }
    
    public static String getAuthName(HttpServletRequest request) {
        String auth = request.getHeader("authorization");
        if (auth == null) {
            return null;
        } else {
            String[] parts = auth.split(" ", 2);
            String authtype = parts[0];
            String credentials = parts[1];
            try {
                String decode = new Base64Decoder(credentials).processString();
                String[] userpass = decode.split(":", 2);
                return userpass[0];
            } catch (Base64FormatException ex) {
                return null;
            }
        }
    }
}
