/*
 * SessionFilter.java
 *
 * Created on 01 April 2005, 21:26
 */

package jim.util;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;


/**
 * A filter to check whether there is a valid session in existence (i.e. it has not timed out).
 * @author briggsj
 */
public class SessionFilter implements Filter {
    
    /** Creates a new instance of SessionFilter */
    public SessionFilter() {
        super();
    }
    
    /**
     * Destroy object
     */    
    public void destroy() {
    }
    
    String badsessionURL = "/pages/badsession.jsp";
    
    /**
     * Implement the session filter
     * @param servletRequest Request
     * @param response Response
     * @param chain Chain
     * @throws IOException exception
     * @throws ServletException exception
     */    
    public void doFilter(ServletRequest servletRequest,
    ServletResponse response,
    FilterChain chain) throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest)servletRequest;
        HttpSession session = request.getSession(false);
        if (session == null || isInvalid(session, request, response)) {
            session = request.getSession();
            int maxMins = session.getMaxInactiveInterval() / 60; //result comes in seconds
            request.setAttribute("sessionMaxInactive", Integer.toString(maxMins));
            RequestDispatcher rd = request.getRequestDispatcher(badsessionURL);
            rd.forward(request, response);
        } else {
            chain.doFilter(request, response);
        }
    }
    
    /**
     * Initialise the session filter
     * If the initialisation parameter "forwardPage" is set, it is used as the URL to forward to if the session is invalid
     * @param cfg the configuration object
     * @throws ServletException exception
     */    
    public void init(FilterConfig cfg) throws ServletException {
        String forwardPage = cfg.getInitParameter("forwardPage");
        if (forwardPage != null) {
            badsessionURL = forwardPage;
        }
    }
    
    /**
     * Default implementation of method to determine whether a session is valid or not.
     * Always returns false.
     * Suitable to be overridden with a more specific test.
     * @param session session object
     * @param request request object
     * @param response response object
     * @return whether the session is invalid
     */    
    protected boolean isInvalid (HttpSession session, ServletRequest request, ServletResponse response) {
        return false;
    }
    
}
