/*
 * PumsBaseAction.java
 *
 * Created on 17 August 2004, 16:17
 */

package jim.util.struts;

import java.sql.Connection;
import java.sql.SQLException;
import javax.sql.DataSource;
import org.apache.struts.action.*;
import javax.servlet.http.*;
import java.io.IOException;
import javax.servlet.ServletException;
import jim.common.*;


/**
 * Things to be done in all actions in all JimApp applications
 * @author briggsj
 */
public abstract class JimBaseAction extends Action {
    
    /**
     * Creates a new instance of JimBaseAction
     */
    public JimBaseAction() {
        super();
    }
    
    /**
     * Check to see whether this action can be performed by the user
     * @param su All the useful info
     * @return whether or not the user is allowed to perform this action
     */
    protected boolean isRoleOK(StrutsUtils su) {
        ActionMapping mapping = su.getMapping();
        HttpServletRequest request = su.getRequest();
        
        String roles[] = mapping.getRoleNames();
        if ((roles == null) || (roles.length < 1)) {
            Trace.print("No roles specified for " + mapping.getPath());
            return true; // OK, if no roles specified (default)
        }
        
        for (int i = 0; i < roles.length; i++) {
            if (request.isUserInRole(roles[i])) {
                Trace.print("User " + request.getRemoteUser() + " has role " + roles[i]);
                return true;
            }
        }
        Trace.print("User " + request.getRemoteUser() + " has no required role: " + roles.toString());
        return false;
    }
    
    
    
    public synchronized final ActionForward execute(
            ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response) throws IOException, ServletException {
        Trace.setLevel(1);
        Trace.print(mapping.getPath());
        
        StrutsUtils su = new StrutsUtils();
        BaseApplication app = getApplication(request);
        su.setApp(app);
        JimActionErrors errors = new JimActionErrors();
        JimActionErrors messages = new JimActionErrors();
        su.setErrors(errors);
        su.setMessages(messages);
        su.setMapping(mapping);
        su.setForm(form);
        su.setRequest(request);
        su.setResponse(response);
        su.setServletContext(this.getServlet().getServletContext());
        
        ActionForward af;
        try {
            if (isRoleOK(su)) {
                af = doExecute(su);
                if (af == null) {
                    ActionMapping m = su.getMapping();
                    throw new JimFatalException("No ActionForward returned by " + m.getType() + " (" + m.getPath() + ")");
                }
            } else {
                errors.addGlobal("errors.notinrole", request.getRemoteUser());
                af = mapping.findForward("notinrole");
            }
        } catch (JimFatalException e) {
            errors.addException(e);
            Throwable t = e.getCause();
            if (t == null) {
                t = e;
            } else {
                errors.addException(t);
            }
            af = mapping.findForward("fatal");
        }
        saveErrors(request, (ActionMessages)errors);
        saveMessages(request, messages);
        return af;
    }
    
    protected BaseApplication getApplication(HttpServletRequest request) {
        BaseApplication app;
        HttpSession session = request.getSession();
        
        app = (BaseApplication)session.getAttribute(JimConstants.APPLICATION);
        if (app == null) {
            app = createApplication();
            session.setAttribute(JimConstants.APPLICATION, app);
        }
        return app;
    }
    
    abstract protected BaseApplication createApplication();
    
    abstract public ActionForward doExecute(StrutsUtils su) throws IOException, ServletException, JimFatalException;
    
}