/*
 * StrutsUtils.java
 *
 * Created on 09 November 2004, 15:15
 */

package jim.util.struts;

/**
 *
 * @author  briggsj
 */

import java.sql.Connection;
import java.sql.SQLException;
import javax.servlet.ServletContext;
import javax.servlet.http.*;
import jim.common.*;
import jim.util.RequestUtils;
import jim.util.SqlUtils;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.struts.action.*;

/**
 * Utility class to group together various useful struts-related operations
 */
public class StrutsUtils {
    
    /**
     * Holds value of property errors.
     */
    private JimActionErrors errors;
    
    /**
     * Holds value of property request.
     */
    private HttpServletRequest request;
    
    /**
     * Holds value of property mapping.
     */
    private ActionMapping mapping;
    
    private HttpServletResponse response;
    
    private ActionForm form;
    
    private BaseApplication app;
    
    /**
     * Holds value of property messages.
     */
    private JimActionErrors messages;
    
    private Connection conn = null;
    
    /** Creates a new instance of StrutsUtils */
    public StrutsUtils() {
    }
    
    /**
     * Getter for property app.
     * @return Value of property app.
     */
    public BaseApplication getApp() {
        return app;
    }
    
    /**
     * Setter for property app.
     * @param app New value of property app.
     */
    public void setApp(BaseApplication app) {
        this.app = app;
    }
    
    /**
     * Getter for property conn.
     * If a connection has not already been made, call the application's getConnection method to establish one.
     * @return Value of property conn.
     * @throws jim.common.JimFatalException if a connection can't be established
     */
    public Connection getConn() throws JimFatalException {
        if (conn == null) {
            try {
                conn = app.getConnection();
            } catch (SQLException e) {
                throw new JimFatalException("Cannot get a database connection", e);
            }
        }
        return conn;
    }
    
    public void closeConn() throws JimFatalException {
        if (conn != null) {
            Trace.print("Close connection");
            try {
                conn.close();
            } catch (SQLException e) {
                throw new JimFatalException("Cannot close db", e);
            }
            conn = null;
        }
    }
    
    private SqlUtils sql = null;
    
    public SqlUtils getSqlUtils() throws JimFatalException {
        if (sql == null) {
            sql = new SqlUtils(getConn());
        }
        return sql;
    }
    
    public void closeSql() throws JimFatalException {
        closeConn();
        if (sql != null) {
            sql = null;
        }
    }
    
    /**
     * Getter for property errors.
     * @return Value of property errors.
     */
    public JimActionErrors getErrors() {
        return errors;
    }
    
    /**
     * Setter for property errors.
     * @param errors New value of property errors.
     */
    public void setErrors(JimActionErrors errors) {
        this.errors = errors;
    }
    
    /**
     * Getter for property form.
     * @return Value of property form.
     */
    public ActionForm getForm() {
        return form;
    }
    
    /**
     * Setter for property form.
     * @param form New value of property form.
     */
    public void setForm(ActionForm form) {
        this.form = form;
    }
    
    /**
     * Getter for property mapping.
     * @return Value of property mapping.
     */
    public ActionMapping getMapping() {
        return mapping;
    }
    
    /**
     * Setter for property mapping.
     * @param mapping New value of property mapping.
     */
    public void setMapping(ActionMapping mapping) {
        this.mapping = mapping;
    }
    
    /**
     * Getter for property request.
     * @return Value of property request.
     */
    public HttpServletRequest getRequest() {
        return request;
    }
    
    /**
     * Setter for property request.
     * @param request New value of property request.
     */
    public void setRequest(HttpServletRequest request) {
        this.request = request;
    }
    
    /**
     * Getter for property response.
     * @return Value of property response.
     */
    public HttpServletResponse getResponse() {
        return response;
    }
    
    /**
     * Setter for property response.
     * @param response New value of property response.
     */
    public void setResponse(HttpServletResponse response) {
        this.response = response;
    }
    
    /**
     * Copy a bean.
     * @param bean the destination bean
     * @param form the source form
     * @throws jim.common.JimFatalException if the bean can't be copied
     */
    public void copyBean(Object bean, ActionForm form) throws JimFatalException {
        try {
            BeanUtils.copyProperties(bean, form);
        } catch (Exception e) {
            errors.addException(e);
            throw new JimFatalException("Cannot copy bean", e);
        }
    }
    
    /**
     * Begin a database transaction.
     * @throws jim.common.JimFatalException if anything goes wrong
     */
    public void beginTransaction() throws JimFatalException {
        Trace.print("Begin transaction");
        try {
            this.getConn().setAutoCommit(false);
        } catch (Exception e) {
            throw new JimFatalException("Cannot set AutoCommit to false", e);
        }
    }
    
    /**
     * End a database transaction.
     * If there are no errors, commit the database, otherwise rollback.
     * @throws jim.common.JimFatalException if anything goes wrong
     * @return ActionForward
     */
    public ActionForward endTransaction() throws JimFatalException {
        ActionForward af = null;
        try {
            if (errors.isEmpty()) {
                Trace.print("End transaction: Commit database");
                this.getConn().commit();
                af = success();
            } else {
                Trace.print("End transaction: Rollback database");
                this.getConn().rollback();
                af = failure();
            }
        } catch (Exception e) {
            throw new JimFatalException("Cannot end transaction", e);
        } finally {
            this.closeConn();
        }
        return af;
    }
    
    /**
     * Return the appropriate ActionForward for success or failure depending on whether there have been any errors reported.
     * @return ActionForward
     */
    public ActionForward successOrFailure() {
        if (errors.isEmpty()) {
            return success();
        } else {
            return failure();
        }
    }
    
    /**
     * Return the success ActionForward.
     * @return ActionForward
     */
    public ActionForward success() {
        return mapping.findForward("success");
    }
    
    /**
     * Return the failure ActionForward.
     * @return ActionForward
     */
    public ActionForward failure() {
        return mapping.findForward("failure");
    }
    
    /**
     * Close database connection.
     * @throws java.lang.Throwable if anything goes wrong
     */
    protected void finalize() throws Throwable {
        this.closeConn();
        super.finalize();
    }
    
    /**
     * Getter for property messages.
     * @return Value of property messages.
     */
    public JimActionErrors getMessages() {
        return this.messages;
    }
    
    /**
     * Setter for property messages.
     * @param messages New value of property messages.
     */
    public void setMessages(JimActionErrors messages) {
        this.messages = messages;
    }
    
    /**
     * return the specified request parameter.
     * @param name name of parameter
     * @return String
     */
    public String getParameter(String name) {
        return RequestUtils.getParameter(request, name);
    }
    
    /**
     * Return the specified request parameter, or the specified default value.
     * @param name of parameter
     * @param dflt default value
     * @return String
     */
    public String getParameter(String name, String dflt) {
        return RequestUtils.getParameter(request, name, dflt);
    }
    
    /**
     * Return the specified request parameter as an int.
     * @param name of parameter
     * @param dflt default value to use if missing
     * @return int
     */
    public int getParameter(String name, int dflt) {
        return RequestUtils.getParameter(request, name, dflt);
    }
    
    /**
     * Request scope
     */
    static public final int REQUEST_SCOPE = 2;
    /**
     * Session scope
     */
    static public final int SESSION_SCOPE = 3;
    /**
     * Application (servlet context) scope
     */
    static public final int APPLICATION_SCOPE = 4;
    
    /**
     * Holds value of property servletContext.
     */
    private ServletContext servletContext;
    
    /**
     * Set the specified attribute in the specified scope.
     * @param name of attribute
     * @param value to use
     * @param scope desired scope
     */
    public void setAttribute(String name, Object value, int scope ) {
        if (scope == REQUEST_SCOPE) {
            request.setAttribute(name, value);
        } else if (scope == SESSION_SCOPE) {
            HttpSession session = request.getSession();
            session.setAttribute(name, value);
        } else if (scope == APPLICATION_SCOPE) {
            servletContext.setAttribute(name, value);
        }
    }
    
    /**
     * Get the specified attribute from the specified scope.
     * @param name of attribute
     * @param scope scope to use
     * @return Object
     */
    public Object getAttribute(String name, int scope) {
        Object item = null;
        if (scope == REQUEST_SCOPE) {
            item = request.getAttribute(name);
        } else if (scope == SESSION_SCOPE) {
            HttpSession session = request.getSession();
            item = session.getAttribute(name);
        } else if (scope == APPLICATION_SCOPE) {
            item = servletContext.getAttribute(name);
        }
        return item;
    }
    
    /**
     * Find the specified attribute in any scope.
     * Tries request, session and application in that order.
     * @param name of attribute
     * @return the found object
     */
    public Object findAttribute(String name) {
        Object item = getAttribute(name, REQUEST_SCOPE);
        if (item == null) {
            item = getAttribute(name, SESSION_SCOPE);
        }
        if (item == null) {
            item = getAttribute(name, APPLICATION_SCOPE);
        }
        return item;
    }
    
    /**
     * Getter for property servletContext.
     * @return Value of property servletContext.
     */
    public ServletContext getServletContext() {
        return this.servletContext;
    }
    
    /**
     * Setter for property servletContext.
     * @param servletContext New value of property servletContext.
     */
    public void setServletContext(ServletContext servletContext) {
        this.servletContext = servletContext;
    }
    
    /**
     * Getter for property session.
     * @return Value of property session.
     */
    public HttpSession getSession() {
        return this.getRequest().getSession();
    }
}